@@ -6,7 +6,7 @@ Build debian packages out of a configuration file.
 Lint command will check the configuration file and output the list of packages to be built.
 
 Usage:
-    trebuchet build <config_file> [--output DEBS_PATH] [--build BUILD_PATH] [--arch ARCH] [--pip-options PIP-OPTIONS] [--app-version APP_VERSION] [--env-version ENV_VERSION] [--service-version SERVICE_VERSION] [--dry-run] [--web-callback URL] [--extra-description MESSAGE]
+    trebuchet build <config_file> [--output DEBS_PATH] [--build BUILD_PATH] [--arch ARCH] [--pip-options PIP-OPTIONS] [--app-version APP_VERSION] [--env-version ENV_VERSION] [--service-version SERVICE_VERSION] [--static-version STATIC_VERSION] [--dry-run] [--web-callback URL] [--extra-description MESSAGE]
     trebuchet develop <config_file> [--build BUILD_PATH] [--arch ARCH] [--pip-options PIP-OPTIONS] [--app-version APP_VERSION] [--env-version ENV_VERSION] [--service-version SERVICE_VERSION] [--extra-description MESSAGE]
     trebuchet lint <config_file>
     trebuchet -h | --help
@@ -22,6 +22,7 @@ Options:
     --app-version APP_VERSION Option for versioning of the application/configuration package. Default is "1.0.0".
     --env-version ENV_VERSION Option for versioning of the environment package. Default is "1.0.0".
     --service-version SERVICE_VERSION Option for versioning of the service packages. Default is "1.0.0".
+    --static-version STATIC_VERSION Option for versioning of the static files packages. Default is "1.0.0".
     --dry-run               Do NOT actually build the packages. Only output the configuration.
     --web-callback URL      Make a HTTP POST request to URL after each package is built with some information related to the build.
     --extra-description MESSAGE Extra description for all the packages built.
@@ -84,7 +85,8 @@ def main():
     # We check if the configuration file is ok.
     elif arguments['lint']:
 
-        check_config(project_config)
+        pkg_list = check_config(project_config)
+        print "Packages to be built: " + ",".join(pkg_list)
 
 
 if __name__ == '__main__':
